/*
 * Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: amingamingaming <wangyiming01@kylinos.cn>
 *
 */

import QtQuick 2.15
import QtGraphicalEffects 1.15
import org.ukui.quick.items 1.0
import org.ukui.quick.platform 1.0

Rectangle {
    //背景颜色
    property var backgroundColor : GlobalTheme.windowActive;
    property var borderColor :  GlobalTheme.baseActive
    property real borderAlpha: 1.0;
    border.width: 0;
    border.color : borderColor.pureColor;
    clip: true;
    LinearGradient {
        id : linerGra
        anchors.fill: parent
        readonly property var points : getGradientPoints(backgroundColor.gradientDeg, width, height);
        readonly property color colorStart: backgroundColor.gradientStart;
        readonly property color colorEnd: backgroundColor.gradientEnd;
        readonly property color colorBack: backgroundColor.gradientBackground;

        start: Qt.point(points[0] * width, points[1] * height)
        end: Qt.point(points[2] * width, points[3] * height)
        visible: backgroundColor.colorType === DtColorType.Gradient
        gradient : Gradient {
            GradientStop { position : 0.0; color: backgroundColor.colorType === DtColorType.Gradient ? mixBackgroundColor(linerGra.colorStart.r,linerGra.colorStart.g,linerGra.colorStart.b,linerGra.colorStart.a,linerGra.colorBack.r,linerGra.colorBack.g,linerGra.colorBack.b,linerGra.colorBack.a) : backgroundColor.pureColor}
            GradientStop { position : 1.0; color: backgroundColor.colorType === DtColorType.Gradient ? mixBackgroundColor(linerGra.colorEnd.r,linerGra.colorEnd.g,linerGra.colorEnd.b,linerGra.colorEnd.a,linerGra.colorBack.r,linerGra.colorBack.g,linerGra.colorBack.b,linerGra.colorBack.a) : backgroundColor.pureColor}
        }
    }
    color:backgroundColor.pureColor

    //根据角度计算渐变开始和结束位置
    function getGradientPoints(angle) {
        var radians = angle * Math.PI / 180;
        var startX = 0.5 - Math.cos(radians) * 0.5;
        var startY = 0.5 - Math.sin(radians) * 0.5;
        var endX = 0.5 + Math.cos(radians) * 0.5;
        var endY = 0.5 + Math.sin(radians) * 0.5;
        return [startX, startY, endX, endY];
    }

    //叠加背景色
    function mixBackgroundColor(r1,g1,b1,a1,r2,g2,b2,a2) {
        var r = r1 * a1 + r2 * a2 * (1 - a1);
        var g = g1 * a1 + g2 * a2 * (1 - a1);
        var b = b1 * a1 + b2 * a2 * (1 - a1);
        var a = a1 + a2 * (1 - a1);
        return Qt.rgba(r,g,b,a);
    }
}
